<?php

namespace Drupal\cforge_address;

use Drupal\address\Event\AddressFormatEvent;
use Drupal\address\Event\AddressEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Rewrites the french address format
 */
class AddressSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[AddressEvents::ADDRESS_FORMAT] = [['rewriteAddress']];
    return $events;
  }

  /**
   * Ensure every address format has %dependentLocality, sticking it on the end
   * if necessary, that that field is required, and called neighbourhood
   */
  public function rewriteAddress(AddressFormatEvent $event) {
    $addressFormat = $event->getDefinition();
    $pattern = $addressFormat['format'];
    if ($addressFormat['country_code'] == 'FR') {
      $addressFormat['format'] = "%addressLine1
%addressLine2
%dependentLocality
%postalCode %locality %administrativeArea";
      $addressFormat['uppercase_fields'][] = 'postalCode';
      $addressFormat['administrative_area_type'] = 'department';
    }
    if (!strpos($addressFormat['format'], '%dependentLocality')) {
      $addressFormat['format'] .= "\n%dependentLocality";
    }

    if (strpos($addressFormat['format'], '%givenName %familyName') == FALSE) {
       $addressFormat['format'] = "%givenName %familyName\r\n".$addressFormat['format'];
    }
    $addressFormat['dependent_locality_type'] = 'neighborhood';
    $addressFormat['required_fields'] = ['dependentLocality', 'givenName'];
    $event->setDefinition($addressFormat);
  }

}
