<?php

namespace Drupal\cforge_gdpr;

use Drupal\user\Entity\User;
use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form builder for GDPR consent
 */
class ConsentForm extends FormBase implements ContainerInjectionInterface {

  protected $userData;

  function __construct($user_data_store) {
    $this->userData = $user_data_store;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('user.data')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_gdpr_consent';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    return array(
      '#title' => t('Do you accept the GDPR terms and conditions which you should have received by mail? Choose ONE of the following:'),
      'agree' => array(
        '#type' => 'item',
        '#markup' => t('I agree that my data may be shared with other members. I promise to use the data accessible on this site only within the context of the activities of this platform.'),
        '#weight' => 1
      ),
      'yes' => array(
        '#type' => 'submit',
        '#value' => t('I agree'),
        '#weight' => 2
      ),
      'disagree' => array(
        '#type' => 'item',
        '#markup' => t("I don't quite understand. I am not sure. I refuse. I wish to be contacted by an administrator."),
        '#weight' => 3
      ),
      'no' => array(
        '#type' => 'submit',
        '#value' => t("I do not agree"),
        '#weight' => 4
      )
    );
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
     $account = User::load(\Drupal::currentUser()->id());
     $accepted = $form_state->getTriggeringElement()['#parents'][0] == 'yes';
     $this->userData->set('cforge_gdpr', $account->id(), 'consent', $accepted);
     if ($accepted) {
       if ($account->hasRole(RID_TRADER)) {
         $account->addRole(RID_TRADER);
         $account->save();
       }
       drupal_set_message(t('Thanks for agreeing to the GDPR conditions. You can now continue to use the site normally.'));

       $url = Url::fromUserInput('/gdpr');
       $form_state->setRedirectUrl($url);
     }
     else {
       if (!$account->hasRole(RID_TRADER)) {
         $account->removeRole(RID_TRADER);
         $account->save();
       }
       // Mail all the members of the committee about this event.
       foreach (gdpr_get_committee() as $account) {
         \Drupal::service('plugin.manager.mail')->mail(
           'cforge_gdpr',
           'gdpr_no',
           $account->mail,
           user_preferred_language($account),
           ['user' => $user, 'recipient' => $account]
         );
       }
       $form_state->setRedirect('cforge_gdpr.declined');
       user_logout();
     }
  }

}
