<?php

namespace Drupal\cforge_gdpr;

use Drupal\user\Entity\User;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\Core\Controller\ControllerBase;


class PageController extends ControllerBase {

  /**
   * Get a list organisation types.
   *
   * @return array
   */
  static function orgTypes() : array {
    return [
      'informal' => t('Informal'),
      'nonprofit' => t('Nonprofit'),//'Association de Fait'
      'company' => t('Company'),// 'ASBL/Loi de 1901'
    ];
  }

  /**
   * Get a list communication channels
   *
   * @return array
   */
  static function channelNames() : array {
    return [
      'cforge_mail' => t("This site's mailing list"),
      'other_mail' => t('Some other mailing list, such as Yahoo, or Google groups'),
      'dropbox' => t('Dropbox or similar'),
      'paper' => t("Via post, or paper taken from this site's member list"),
      'twitter' => t('Twitter')
    ];
  }

  /**
   * Page if the user declines to accept GDPR.
   *
   * @return type
   */
  function declined() : array {
    return [
      '#markup' => $this->t("Either log in again and click 'agree' or wait for a member of the committee to contact you.")
    ];
  }

  /**
   * Page showing GDPR statement.
   *
   * @return array
   */
  function statement() : array {
    $gdpr_settings = $this->config('cforge_gdpr.settings');
    $committee = [];
    foreach (gdpr_get_committee() as $account) {
      $committee[$account->getUsername()] = t('Committee member');
    }
    return [
      '#theme' => 'gdpr_statement',
      '#orgtype' => $gdpr_settings->get('orgtype'),
      '#treasurer' => User::load($gdpr_settings->get('treasurer')),
      //Cannot use empty array elements in arrays
      '#channels' => array_keys($gdpr_settings->get('channels')),
      '#responsible' => [
        $this->config('system.site')->get('name') => $committee
      ],
      '#httphost' => $_SERVER['HTTP_HOST'],
      '#site_mail' => \Drupal::config('system.site')->get('mail'),
      '#host_org' => Link::fromTextAndUrl(
        $this->config('system.site')->get('name'),
        Url::fromUri($_SERVER['REQUEST_SCHEME'].'://'.$_SERVER['HTTP_HOST'])
      ),
      '#editor' => User::load($gdpr_settings->get('editor', 1)),
      '#edited' => 0, //$gdpr_settings->get('edited', 0)
    ];
  }


  function statementTitle() : \Drupal\Component\Render\MarkupInterface {
    return $this->t(
      'GDPR statement for %site_name',
      ['%site_name' => $this->config('system.site')->get('name')]
    );
  }
}