<?php

namespace Drupal\cforge_gdpr;

use Drupal\cforge_gdpr\PageController;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form builder for GDPR consent
 */
class SettingsForm extends FormBase implements ContainerInjectionInterface {

  protected $currentUser;
  protected $time;

  function __construct($current_user, $time_service) {
    $this->currentUser = $current_user;
    $this->time = $time_service;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('datetime.time')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_gdpr_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $settings = $this->config('cforge_gdpr.settings');
    $form['subtitle'] = [
      '#type'=> 'item',
      '#markup' => $this->t('Edit the GDPR document for your association.'),
  // Ce webform vous pemet de créér le document RGPD pour votre Sel/Association.
    ];
    $form['orgtype'] = array(
      '#title' => $this->t("Type of organisation"),//"Type de Sel ou d'association"
      '#type' => 'radios',
      '#options' => PageController::orgTypes(),
      '#default_value' => $settings->get('orgtype', 'nonprofit'),
      '#weight' => 1
    );

    if ($committee = gdpr_get_committee()) {
      foreach ($committee as $uid => $account) {
        $options[$uid] = $account->getDisplayName();
      }
      $form['treasurer'] = [
        '#title' => $this->t('Treasurer'), // Trésorier
        '#type' => 'select',
        '#options' => $options,
        '#default_value' => $settings->get('treasurer', key($options)),
        '#weight' => 2
      ];
    }
    else {
      $form['treasurer']= [
        '#type' => 'value',
        '#value' => 1
      ];
    }
    $form['channels'] = array(
      '#title' => $this->t('Communication channels used'),
      '#type' => 'checkboxes',
      '#options' => PageController::channelNames(),
      '#default_value' => $settings->get('channels'),
      '#weight' => 4,
      'cforge_mail' => array(
        '#value' => 'cforge_mail',
        '#disabled' => TRUE
      )
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => 'Save',
      '#weight' => 5
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory()->getEditable('cforge_gdpr.settings')
      ->set('orgtype', $form_state->getValue('orgtype'))
      ->set('treasurer', $form_state->getValue('treasurer'))
      ->set('channels', array_filter($form_state->getValue('channels')))
      ->set('editor', \Drupal::currentUser()->id())
      ->set('edited', \Drupal::time()->getCurrentTime())
      ->save();
    $form_state->setRedirect('cforge_gdpr.statement');
  }

}
